@@ -1,6 +1,8 @@
 import numpy as np
 import matplotlib.pyplot as plt
+from scipy import ndimage
 from scipy.interpolate import griddata
+from scipy.interpolate import Rbf, LinearNDInterpolator, interp2d
 from scipy.spatial import cKDTree as KDTree
 from mpl_toolkits.axes_grid1 import make_axes_locatable
 
@@ -64,6 +66,41 @@ def deg2km(lat):
     # earth radius: 6371 km
     return 6371 * np.cos(lat) * 2* np.pi / 360
 
+def grid2points(arr, lons, lats, lon_pnts, lat_pnts, order = 1):
+    # Python array starts from upper left (longitude ascending but latitude descending)
+    # check orders:
+    if lats[-1] > lats[0]:
+        lats = lats[::-1]
+        arr = arr[::-1, :]
+    if lons[-1] < lons[0]:
+        lons = lons[::-1]
+        arr = arr[:, ::-1]
+    # print(lats)
+    # print(lons)
+    # print(arr)
+    def lonlat2xy(lonlats, lonlat):
+        xy = (lonlat - lonlats[0]) / (lonlats[-1] - lonlats[0]) * (len(lonlats) - 1)
+        return xy
+    
+    map_lons = [lonlat2xy(lons, lon) for lon in lon_pnts]
+    map_lats = [lonlat2xy(lats, lat) for lat in lat_pnts]
+    # print(map_lons)
+    # print(map_lats)
+
+    # map_lat = (lat - lats[0]) / (lats[-1] - lats[0]) * (len(lats) - 1)
+    # # print(map_lat)
+
+    # map_lon = (lon - lons[0]) / (lons[-1] - lons[0]) * (len(lons) - 1)
+    # # print(map_lon)
+
+    return ndimage.map_coordinates(arr, [map_lats, map_lons], order = order)
+
+def grid2points2(arr, lons, lats, lon_pnts, lat_pnts, missing = -9999, method = 'linear'):
+    arr[np.where(arr == missing)] = np.nan
+    lonlon, latlat = np.meshgrid(lons, lats)
+    data = griddata((latlat.ravel(), lonlon.ravel()), arr.ravel(), (lat_pnts, lon_pnts), method = method)
+    return data
+
 def points2grid(data, lons, lats, missing = -9999, method = 'nearest'):
     uniqueLats = np.unique(lats)
     uniqueLons = np.unique(lons)
@@ -75,6 +112,14 @@ def points2grid(data, lons, lats, missing = -9999, method = 'nearest'):
 
     return arr, uniqueLons, uniqueLats
 
+def points2points(data, lons_orig, lats_orig, lons_tar, lats_tar):
+    """
+    data, lons_orig, lats_orig, lons_tar, lats_tar are all vectors (1D array)
+    """
+    interp = LinearNDInterpolator(list(zip(lons_orig, lats_orig)), data)
+    # interp = Rbf(lons_orig, lats_orig, arr,function='linear')
+    return interp(lons_tar, lats_tar)
+
 def latex_float(f):
     if (f < 0.01) or (f > 100):
         float_str = "{0:.2e}".format(f)
